package plegatfem2d.objects;

import java.awt.Color;
import java.awt.Graphics;
import plegatfem2d.PFEM2DGuiPanel;

/**
 * Classe définissant un objet graphique de type "ligne", reliant deux points.
 * @author Jean-Michel BORLOT
 */
public class PFEM2DLine implements IPFEM2DDrawableObject, IPFEM2DCurve {

    private PFEM2DPoint pt1, pt2;   // les deux points définissant la ligne 
    private int id;                 // l'identité de la ligne
    private boolean visible;        // l'état de visibilité de la ligne
    
    /**
     * Constructeur
     * @param id identité
     * @param pt1 point n°1
     * @param pt2 point n°2
     */
    public PFEM2DLine(int id, PFEM2DPoint pt1, PFEM2DPoint pt2) {
        this.pt1 = pt1;
        this.pt2 = pt2;
        this.id = id;
        this.visible = true;
    }

    /**
     * Définit l'identité de la ligne
     * @param id l'identité de la ligne
     */
    public void setId(int id) {
        this.id = id;
    }

    /**
     * Renvoie le premier point
     * @return le premier point
     */
    public PFEM2DPoint getPt1() {
        return pt1;
    }

    /**
     * Définit le premier point
     * @param pt1 le premier point
     */
    public void setPt1(PFEM2DPoint pt1) {
        this.pt1 = pt1;
    }

    /**
     * Renvoie le second point
     * @return le second point
     */
    public PFEM2DPoint getPt2() {
        return pt2;
    }

    /**
     * Définit le second point
     * @param pt2 le second point
     */
    public void setPt2(PFEM2DPoint pt2) {
        this.pt2 = pt2;
    }

    /**
     * Dessine l'objet graphique
     * @param g l'objet Graphics sur lequel dessiner
     * @param panel l'objet PFEM2DGuiPanel sur lequel dessiner
     */
    @Override
    public void draw(Graphics g, PFEM2DGuiPanel panel) {

        if (this.isVisible()) {
            int xloc1 = panel.getLocalCoordX(this.pt1.getX());
            int yloc1 = panel.getLocalCoordY(this.pt1.getY());

            int xloc2 = panel.getLocalCoordX(this.pt2.getX());
            int yloc2 = panel.getLocalCoordY(this.pt2.getY());

            g.setColor(Color.yellow);
            g.drawLine(xloc1, yloc1, xloc2, yloc2);
        }
    }

    /**
     * Définit la visibilité de l'objet graphique
     * @param flag true si l'objet est visible, false s'il ne l'est pas
     */
    @Override
    public void setVisible(boolean flag) {
        this.visible = flag;
    }

    /**
     * Renvoie l'état de visibilité de l'objet graphique
     * @return true si l'objet est visible, false s'il ne l'est pas
     */
    @Override
    public boolean isVisible() {
        return this.visible;
    }

    /**
     * Renvoie l'identité de l'objet graphique
     * @return l'identité de l'objet graphique
     */
    @Override
    public String getId() {
        return "Line " + this.id;
    }

    /**
     * Renvoie le point de départ de la courbe
     * @return le point de départ de la courbe
     */
    @Override
    public PFEM2DPoint getStartPoint() {
        return this.pt1;
    }

    /**
     * Renvoie le point d'arrivée de la courbe
     * @return le point d'arrivée de la courbe
     */
    @Override
    public PFEM2DPoint getEndPoint() {
        return this.pt2;
    }

    /**
     * Renvoie la boite englobante de l'objet graphique
     * @return un tableau de 4 double: Xmin, Xmax, Ymin, Ymax
     */
    @Override
    public double[] getBoundingBox() {
        
        double data[]=new double[4];
        
        data[0]=Math.min(this.pt1.getX(),this.pt2.getX());
        data[1]=Math.max(this.pt1.getX(),this.pt2.getX());
        data[2]=Math.min(this.pt1.getY(),this.pt2.getY());
        data[3]=Math.max(this.pt1.getY(),this.pt2.getY());
        
        return data;
    }

    /**
     * Renvoie la liste des points définissant l'objet graphique
     * @return la liste des points
     */
    @Override
    public PFEM2DPoint[] getPoints() {
        return new PFEM2DPoint[]{this.pt1,this.pt2};
    }
}
